package com.community.huiju.dao;


import com.community.huiju.model.TaUserVerify;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface TaUserVerifyMapper {
    int deleteByPrimaryKey(Integer id);

    int insert(TaUserVerify record);

    int insertSelective(TaUserVerify record);

    TaUserVerify selectByPrimaryKey(Integer id);

    int updateByPrimaryKeySelective(TaUserVerify record);

    int updateByPrimaryKey(TaUserVerify record);

    /**
     * 根据 用户id 查询房产
     * @param userId
     * @return
     */
    List<TaUserVerify> selectByUserId(@Param("userId") Integer userId);

    /**
     * 根据 用户id、小区id 、 期、栋、单元、楼层、户号、审核状态  进行条件查询
     * @param communityId
     * @param phaseId
     * @param buildingId
     * @param unitId
     * @param levelId
     * @param roomNoId
     * @param verifyStatus
     * @return
     */
    List<TaUserVerify> selectCommunityAndAddress(@Param("userId") Integer userId,
                                           @Param("communityId") Integer communityId,
                                           @Param("phaseId") Integer phaseId,
                                           @Param("buildingId") Integer buildingId,
                                           @Param("unitId") Integer unitId,
                                           @Param("levelId") Integer levelId,
                                           @Param("roomNoId") Integer roomNoId,
                                           @Param("verifyStatus") Integer verifyStatus);



    /**
     * 根据 小区id 、 期、栋、单元、楼层、户号, 角色id(业主), 审核通过(值为1) 进行条件查询
     * @param communityId
     * @param phaseId
     * @param buildingId
     * @param unitId
     * @param levelId
     * @param roomNoId
     * @return
     */
    @ResultMap("BaseResultMap")
    @Select("select * from ta_user_verify where community_id=#{communityId}" +
            " and phase_id=#{phaseId} and building_id=#{buildingId}" +
            " and unit_id=#{unitId} and level_id=#{levelId}" +
            " and room_no_id=#{roomNoId} and role_id=1 and verify_status=1")
    TaUserVerify selectCommunityAndAddressAndRoleId(@Param("communityId") Integer communityId,
                                                    @Param("phaseId") Integer phaseId,
                                                    @Param("buildingId") Integer buildingId,
                                                    @Param("unitId") Integer unitId,
                                                    @Param("levelId") Integer levelId,
                                                    @Param("roomNoId") Integer roomNoId);


    /**
     * 根据 用户id、小区id 、 期、栋、单元、楼层、户号、审核状态(不等于2)  进行条件查询
     * @param communityId
     * @param phaseId
     * @param buildingId
     * @param unitId
     * @param levelId
     * @param roomNoId
     * @return
     */
    @ResultMap("BaseResultMap")
    @Select("select * from ta_user_verify where user_id=#{userId} and community_id=#{communityId}" +
            " and phase_id=#{phaseId} and building_id=#{buildingId}" +
            " and unit_id=#{unitId} and level_id=#{levelId}" +
            " and room_no_id=#{roomNoId} and verify_status != 2")
    List<TaUserVerify> selectCommunityAndAddressAndNotVerifyStatus2(@Param("userId") Integer userId,
                                           @Param("communityId") Integer communityId,
                                           @Param("phaseId") Integer phaseId,
                                           @Param("buildingId") Integer buildingId,
                                           @Param("unitId") Integer unitId,
                                           @Param("levelId") Integer levelId,
                                           @Param("roomNoId") Integer roomNoId);

    @ResultMap("BaseResultMap")
    @Select("select * from ta_user_verify where community_id=#{communityId}" +
            " and verify_status =  1 and id in (1120,1122,1128,1133,1134,1119)")
    List<TaUserVerify> selectByCommunityId(@Param("communityId") Integer communityId);


    /**
     * 查询没有推送默认消息的审核数据
     * @param communityId
     * @return
     */
    @ResultMap("BaseResultMap")
    @Select(
            "SELECT *  FROM " +
            "ta_user_verify uv " +
            "WHERE   uv.verify_status = \"1\" " +
             "and " +
               "uv.community_id = #{communityId} " +
             "and " +
              "uv.id NOT IN (" +
                "SELECT " +
                    "tuv.id  " +
                "FROM " +
                "ta_user_verify tuv  LEFT JOIN tp_message tm ON tuv.id = tm.uuid " +
                "WHERE " +
                    "tm.uuid_type = \"1\" " +
                    "AND tuv.verify_status = \"1\" " +
                    "AND tuv.community_id = #{communityId} " +
                    "AND tm.message_type = \"9\" " +
                    "AND tm.advice_type = \"1\" " +
                    "AND tm.model_type = \"1\" " +
                    "AND tm.source = \"1\" " +
                    "AND tm.`status` = \"1\" " +
                "GROUP BY " +
                "tuv.id  " +
            ") "
    )
    List<TaUserVerify> selectNotMessage(@Param("communityId") Integer communityId);

    /**
     * 查询 这个审核id 是否已经推送过消息
     * @param communityId
     * @param userVerfiyId
     */
    @ResultMap("BaseResultMap")
    @Select("SELECT " +
            "* " +
            "FROM " +
            "ta_user_verify tuv " +
            "LEFT JOIN tp_message tm ON tuv.id = tm.uuid  " +
            "WHERE " +
            "tm.uuid_type = \"1\"  " +
            "AND tuv.verify_status = \"1\"  " +
            "AND tuv.community_id = #{communityId}  " +
            "AND tm.message_type = \"9\"  " +
            "AND tm.advice_type = \"1\"  " +
            "AND tm.model_type = \"1\"  " +
            "AND tm.source = \"1\"  " +
            "AND tm.`status` = \"1\"  " +
            "and tm.uuid = #{userVerfiyId} " +
            "GROUP BY " +
            "tuv.id ")
    TaUserVerify selectChcekNotMessage(@Param("communityId") Integer communityId, @Param("userVerfiyId") Integer userVerfiyId);

    /**
     * 校验这个审核id, 是不是这个房产下面的
     * @param communityId
     * @param phaseId
     * @param buildingId
     * @param unitId
     * @param levelId
     * @param roomNoId
     * @param userVerifyId
     * @return
     */
    TaUserVerify selectChcekInHouseUserVerify(@Param("communityId") Integer communityId, @Param("phaseId") Integer phaseId, @Param("buildingId") Integer buildingId, @Param("unitId") Integer unitId, @Param("levelId") Integer levelId, @Param("roomNoId") Integer roomNoId, @Param("userVerifyId") Integer userVerifyId);
}